@@ -76,6 +76,7 @@ enum {
 	MAGNIFIER_CURSOR_HOTSPOT_PROP,
 	MAGNIFIER_CURSOR_DEFAULT_SIZE_PROP,
 	MAGNIFIER_CROSSWIRE_SIZE_PROP,
+	MAGNIFIER_CROSSWIRE_LENGTH_PROP,
 	MAGNIFIER_CROSSWIRE_CLIP_PROP,
 	MAGNIFIER_CROSSWIRE_COLOR_PROP
 } PropIdx;
@@ -111,6 +112,7 @@ gchar* mag_prop_names[MAGNIFIER_CROSSWIRE_COLOR_PROP + 1] = {
     "CURSOR_HOTSPOT",
     "CURSOR_DEFAULT_SIZE",
     "CROSSWIRE_SIZE",
+    "CROSSWIRE_LENGTH",
     "CROSSWIRE_CLIP",
     "CROSSWIRE_COLOR"
 };
@@ -224,6 +226,27 @@ magnifier_warp_cursor_to_screen (Magnifier *magnifier)
 	}
 }
 
+void
+magnifier_zoom_regions_update_pointer (Magnifier *magnifier)
+{
+	GList *list;
+
+	g_assert (magnifier);
+
+	list = magnifier->zoom_regions;
+	while (list) {
+		/* propagate the expose events to the zoom regions */
+		GNOME_Magnifier_ZoomRegion zoom_region;
+		CORBA_Environment ev;
+		zoom_region = list->data;
+		CORBA_exception_init (&ev);
+		if (zoom_region)
+			GNOME_Magnifier_ZoomRegion_updatePointer (
+				CORBA_Object_duplicate (zoom_region, &ev), &ev);
+		list = g_list_next (list);
+	}
+}
+
 static void
 magnifier_zoom_regions_mark_dirty (Magnifier *magnifier, GNOME_Magnifier_RectBounds rect_bounds)
 {
@@ -280,8 +303,10 @@ magnifier_set_cursor_from_pixbuf (Magnifier *magnifier,
 		g_object_unref (gc);
 		magnifier->priv->cursor = pixmap;
 		magnifier->priv->cursor_mask = mask;
-		xhot_string = gdk_pixbuf_get_option (cursor_pixbuf,"x_hot");
-		yhot_string = gdk_pixbuf_get_option (cursor_pixbuf,"y_hot");
+
+		xhot_string = g_object_get_data (cursor_pixbuf, "x_hot");
+		yhot_string = g_object_get_data (cursor_pixbuf, "y_hot");
+
 		if (xhot_string)
 			magnifier->cursor_hotspot.x = atoi (xhot_string);
 		if (yhot_string)
@@ -429,52 +454,69 @@ magnifier_get_display_rect_bounds (Magnifier *magnifier, GNOME_Magnifier_RectBou
     }
 }
 
+gboolean
+magnifier_full_screen_capable (Magnifier *magnifier)
+{
+	if ((strcmp (magnifier->source_display_name,
+		     magnifier->target_display_name) != 0) ||
+		gmag_gs_use_compositor (magnifier))
+		return TRUE;
+
+	return FALSE;
+}
+
 static void
 magnifier_adjust_source_size (Magnifier *magnifier)
 {
 	GNOME_Magnifier_RectBounds rect_bounds;	
 	gdouble vfract_top, vfract_bottom, hfract_left, hfract_right;
+
 	magnifier_get_display_rect_bounds (magnifier, &rect_bounds, FALSE);
-	hfract_left = (double) (magnifier->target_bounds.x1) / (double) rect_bounds.x2;
-	vfract_top = (double) (magnifier->target_bounds.y1) / (double) rect_bounds.y2;
-	hfract_right = (double) (rect_bounds.x2 - magnifier->target_bounds.x2) / (double) rect_bounds.x2;
-	vfract_bottom = (double) (rect_bounds.y2 - magnifier->target_bounds.y2) / (double) rect_bounds.y2;
+
+	hfract_left = (double) (magnifier->target_bounds.x1) /
+		(double) rect_bounds.x2;
+	vfract_top = (double) (magnifier->target_bounds.y1) /
+		(double) rect_bounds.y2;
+	hfract_right = (double) (rect_bounds.x2 -
+				 magnifier->target_bounds.x2) /
+		(double) rect_bounds.x2;
+	vfract_bottom = (double) (rect_bounds.y2 -
+				  magnifier->target_bounds.y2) /
+		(double) rect_bounds.y2;
+
 	/* we make our 'source' rectangle the largest available subsection which we aren't occupying */
-#ifdef HAVE_COMPOSITE
-	if (!g_getenv ("MAGNIFIER_IGNORE_COMPOSITE")) {
+	if (magnifier_full_screen_capable (magnifier)) {
 		magnifier->source_bounds = rect_bounds;
-	} else
-#endif
-	if (MAX (hfract_left, hfract_right) > MAX (vfract_top, vfract_bottom))  /* vertical split, approximately */
-	{
+	} else if (MAX (hfract_left, hfract_right) >
+		   MAX (vfract_top, vfract_bottom)) {  /* vertical split,
+							* approximately */
 		if (hfract_right > hfract_left) {
-			magnifier->source_bounds.x1 = magnifier->target_bounds.x2;
+			magnifier->source_bounds.x1 =
+				magnifier->target_bounds.x2;
 			magnifier->source_bounds.x2 = rect_bounds.x2;
-		}
-		else 
-		{
+		} else  {
 			magnifier->source_bounds.x1 = rect_bounds.x1;
-			magnifier->source_bounds.x2 = magnifier->target_bounds.x1;
+			magnifier->source_bounds.x2 =
+				magnifier->target_bounds.x1;
 		}
 		magnifier->source_bounds.y1 = rect_bounds.y1;
 		magnifier->source_bounds.y2 = rect_bounds.y2;
-	}
-	else /* more-or-less horizontally split */
-	{
+	} else { /* more-or-less horizontally split */
 		if (vfract_bottom > vfract_top) {
-			magnifier->source_bounds.y1 = magnifier->target_bounds.y2;
+			magnifier->source_bounds.y1 =
+				magnifier->target_bounds.y2;
 			magnifier->source_bounds.y2 = rect_bounds.y2;
-		}
-		else 
-		{
+		} else {
 			magnifier->source_bounds.y1 = rect_bounds.y1;
-			magnifier->source_bounds.y2 = magnifier->target_bounds.y1;
+			magnifier->source_bounds.y2 =
+				magnifier->target_bounds.y1;
 		}
 		magnifier->source_bounds.x1 = rect_bounds.x1;
 		magnifier->source_bounds.x2 = rect_bounds.x2;
 	}
 	g_message ("set source bounds to %d,%d; %d,%d", 
-		   magnifier->source_bounds.x1, magnifier->source_bounds.y1, magnifier->source_bounds.x2, magnifier->source_bounds.y2);
+		   magnifier->source_bounds.x1, magnifier->source_bounds.y1,
+		   magnifier->source_bounds.x2, magnifier->source_bounds.y2);
 }
 
 static void
@@ -897,6 +939,9 @@ magnifier_get_property (BonoboPropertyBag *bag,
 	case MAGNIFIER_CROSSWIRE_SIZE_PROP:
 		BONOBO_ARG_SET_INT (arg, magnifier->crosswire_size);
 		break;
+	case MAGNIFIER_CROSSWIRE_LENGTH_PROP:
+		BONOBO_ARG_SET_INT (arg, magnifier->crosswire_length);
+		break;
 	case MAGNIFIER_CROSSWIRE_CLIP_PROP:
 		BONOBO_ARG_SET_BOOLEAN (arg, magnifier->crosswire_clip);
 		break;
@@ -924,6 +969,12 @@ magnifier_set_property (BonoboPropertyBag *bag,
 	Magnifier *magnifier = user_data;
 	gchar *full_display_string;
 
+	GNOME_Magnifier_RectBounds rect_bounds;
+	rect_bounds.x1 = 0;
+	rect_bounds.y1 = 0;
+	rect_bounds.x2 = 4096;
+	rect_bounds.y2 = 4096;
+
 	switch (arg_id) {
 	case MAGNIFIER_SOURCE_DISPLAY_PROP:
 		full_display_string = BONOBO_ARG_GET_STRING (arg);
@@ -956,21 +1007,15 @@ magnifier_set_property (BonoboPropertyBag *bag,
 			    magnifier->priv->source_drawable = NULL;
 		    }
 
-		    /* If the COMPOSITE extension was not available in the
-		     * previous display the MAGNIFIER_IGNORE_COMPOSITE
-		     * variable is setted. We must unset it to be sure that we
-		     * will make use of this extension if it's present in the
-		     * new display.
-		     */
-		    g_unsetenv ("MAGNIFIER_IGNORE_COMPOSITE");
-
 		    /* attach listeners for DAMAGE, "dirty region", XFIXES
 		     * cursor changes */
 	            magnifier_set_extension_listeners (magnifier, magnifier_get_root (magnifier));
 		    magnifier_get_display_rect_bounds (magnifier, &magnifier->source_bounds, FALSE);
+		    magnifier_adjust_source_size (magnifier);
 		    magnifier_zoom_regions_restore (magnifier, zoom_region_params);
 		    magnifier_warp_cursor_to_screen (magnifier);
 		    gmag_gs_check_set_struts (magnifier);
+		    gmag_gs_reset_overlay_at_idle (magnifier);
 		}
 		DBG(fprintf (stderr, "Set source display: \t%s\n", full_display_string));
 		break;
@@ -1122,11 +1167,20 @@ magnifier_set_property (BonoboPropertyBag *bag,
 	case MAGNIFIER_CROSSWIRE_SIZE_PROP:
 		magnifier->crosswire_size = BONOBO_ARG_GET_INT (arg);
 		DBG (fprintf (stderr, "Setting crosswire size: \t%d\n", magnifier->crosswire_size));
-		/* TODO: notify zoomers */
+		magnifier_zoom_regions_update_pointer (magnifier);
+		break;
+	case MAGNIFIER_CROSSWIRE_LENGTH_PROP:
+		magnifier->crosswire_length = BONOBO_ARG_GET_INT (arg);
+		DBG (fprintf (stderr, "Setting crosswire length: \t%d\n", magnifier->crosswire_length));
+		/* FIXME: The call below forces the repaint of all the screen,
+		 * but I can't figure a better solution right now to the case
+		 * where the crosswire lenght decrease */
+		magnifier_zoom_regions_mark_dirty (magnifier, rect_bounds);
 		break;
 	case MAGNIFIER_CROSSWIRE_CLIP_PROP:
 		magnifier->crosswire_clip = BONOBO_ARG_GET_BOOLEAN (arg);
 		DBG (fprintf (stderr, "Setting crosswire clip: \t%s\n", magnifier->crosswire_clip ? "true" : "false"));
+		magnifier_zoom_regions_update_pointer (magnifier);
 		break;
 	case MAGNIFIER_CROSSWIRE_COLOR_PROP:
 		magnifier->crosswire_color = BONOBO_ARG_GET_LONG (arg);
@@ -1160,6 +1214,41 @@ magnifier_gobject_dispose (GObject *object)
 	BONOBO_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
 }
 
+static
+CORBA_boolean
+impl_magnifier_support_colorblind_filters (Magnifier *magnifier)
+{
+#ifdef HAVE_COLORBLIND
+	return CORBA_TRUE;
+#else
+	return CORBA_FALSE;
+#endif
+}
+
+static void
+impl_magnifier_hide_cursor (PortableServer_Servant servant,
+                        CORBA_Environment *ev)
+{
+        Magnifier *magnifier = MAGNIFIER (bonobo_object_from_servant (servant));
+
+	gmag_gs_hide_cursor (magnifier);
+}
+
+static
+CORBA_boolean
+impl_magnifier_full_screen_capable (PortableServer_Servant servant,
+				    CORBA_Environment * ev)
+{
+	Magnifier *magnifier = MAGNIFIER (bonobo_object_from_servant (servant));
+
+	if ((strcmp (magnifier->source_display_name,
+		     magnifier->target_display_name) != 0) ||
+		gmag_gs_use_compositor (magnifier))
+		return CORBA_TRUE;
+
+	return CORBA_FALSE;
+}
+
 static void
 impl_magnifier_set_source_display (PortableServer_Servant servant,
 				   const CORBA_char *display,
@@ -1396,6 +1485,10 @@ magnifier_class_init (MagnifierClass *klass)
 
 	object_class->dispose = magnifier_gobject_dispose;
 
+	epv->fullScreenCapable = impl_magnifier_full_screen_capable;
+	epv->hideCursor = impl_magnifier_hide_cursor;
+	epv->supportColorblindFilters =
+		impl_magnifier_support_colorblind_filters;
         epv->_set_SourceDisplay = impl_magnifier_set_source_display;
 	epv->_set_TargetDisplay = impl_magnifier_set_target_display;
         epv->_get_SourceDisplay = impl_magnifier_get_source_display;
@@ -1544,6 +1637,15 @@ magnifier_properties_init (Magnifier *magnifier)
 				 "thickness of crosswire cursor, in target pixels",
 				 Bonobo_PROPERTY_READABLE |
 				 Bonobo_PROPERTY_WRITEABLE);
+
+	bonobo_property_bag_add (magnifier->property_bag,
+				 "crosswire-length",
+				 MAGNIFIER_CROSSWIRE_LENGTH_PROP,
+				 BONOBO_ARG_INT,
+				 NULL,
+				 "length of crosswire cursor, in target pixels",
+				 Bonobo_PROPERTY_READABLE |
+				 Bonobo_PROPERTY_WRITEABLE);
 	
 	bonobo_property_bag_add (magnifier->property_bag,
 				 "crosswire-color",
@@ -1606,6 +1708,7 @@ magnifier_init (Magnifier *magnifier)
 	magnifier->cursor_scale_factor = 1.0F;
 	magnifier->cursor_color = 0xFF000000;
 	magnifier->crosswire_size = 1;
+	magnifier->crosswire_length = 0;
 	magnifier->crosswire_color = 0;
 	magnifier->crosswire_clip = FALSE;
 	magnifier->cursor_hotspot.x = 0;
